require File.expand_path(File.dirname(__FILE__) + '/../../spec_helper')
require 'webmock/rspec'
module Mock
class Resource < ActiveResource::Hel::Resource
end
end

describe "A HEL Resource with HTTP cache" do
  before :each do
    ActiveResource::Hel::Resource.cache = ActiveSupport::Cache::MemoryStore.new
    #ActiveResource::Hel::Resource.logger = Logger.new(STDOUT)
    #ActiveResource::Hel::Resource.logger.level = :debug
  end

  def req_url(site, path)
    uri = URI(site.to_s)
    uri.path = path
    uri.to_s
  end

  def stub_auth
    stub_request(:post, req_url(Mock::Resource.site, "/ygg/session/authenticate_by_fqda_and_password")).
      with(
        body: {fqda: Mock::Resource.hel_identity, password: Mock::Resource.hel_secret}
    ).to_return(
	    status: 200,
      body: {
        "id":"089af4a6-d558-40f1-86c4-c4ca3bf03815",
        "authenticated":true,
        "status":"authenticated",
        "started_at":"2019-09-21T10:15:54.000+02:00",
        "auth_credential":"Ygg::Core::Identity::Credential::ObfuscatedPassword",
        "auth_identity":"angelo@intercom.it",
        "auth_method":"fqda_and_password",
        "auth_confidence":"medium",
        "capabilities":["superuser","full_interface","simple_interface","api","installer"],
        "active":true,
        "msg":"Auhtentication successful!",
        "reason":"ok"
      }.to_json
    )
  end

  it 'should request a resource not cached' do
    stub_auth
    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      to_return(status: 200, body: {
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3' 
    }.to_json)

    res = Mock::Resource.find(1)
    expect(res).to be_a Mock::Resource
  end

  it 'should not request a cached resource' do
    stub_auth
    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      to_return(status: 200, body: {
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }.to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )

    res = Mock::Resource.find(1)
    expect(res).to be_a Mock::Resource

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).once

    res = Mock::Resource.find(1)
    expect(res).to be_a Mock::Resource

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).once
  end

  it 'should revalidate a stale cached resource' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      to_return(status: 200, body: {
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }.to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )

    res = Mock::Resource.find(1)
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)
    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).once

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      with(headers: {'If-None-Match' => 'abc123'}).
      to_return(status: 304, body: "{}",
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )
    Timecop.travel(610.seconds.from_now) do
      res = Mock::Resource.find(1)
      expect(res).to be_a Mock::Resource
    end

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).twice
  end

  it 'should refresh a stale cached resoource' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      to_return(status: 200, body: {
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }.to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )

    res = Mock::Resource.find(1)
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).once

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      with(headers: {'If-None-Match' => 'abc123'}).
      to_return(status: 200, body: {
        id: 1,
        field1: 'test-1',
        field2: 'test-2',
        field3: 'test-3'
      }.to_json,
      headers: {
        'ETag' => 'def456',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )
    Timecop.travel(610.seconds.from_now) do
      newres = Mock::Resource.find(1)
      expect(newres).to be_a Mock::Resource
      expect(newres.field1).to eq 'test-1'
      expect(newres.field2).to eq 'test-2'
      expect(newres.field3).to eq 'test-3'
    end

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).twice
  end

  it 'should refresh after max-age seconds' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      to_return(status: 200, body: {
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }.to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )

    res = Mock::Resource.find(1)
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).once

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      with(headers: {'If-None-Match' => 'abc123'}).
      to_return(status: 304, body: "{}",
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )
    
    # no request
    Timecop.travel(590.seconds.from_now) do
      res = Mock::Resource.find(1)
      expect(res).to be_a Mock::Resource
    end

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).once

    # do request
    Timecop.travel(610.seconds.from_now) do
      res = Mock::Resource.find(1)
      expect(res).to be_a Mock::Resource
    end

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).twice
  end

  it 'should refresh every request no-cache resources' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      to_return(status: 200, body: {
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }.to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, no-cache' # pointless, but....
      }
    )

    res = Mock::Resource.find(1)
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).once

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      with(headers: {'If-None-Match' => 'abc123'}).
      to_return(status: 304, body: "{}",
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )
    
    # no request
    Timecop.travel(10.seconds.from_now) do
      res = Mock::Resource.find(1)
      expect(res).to be_a Mock::Resource
    end

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).twice

    # do request
    Timecop.travel(60.seconds.from_now) do
      res = Mock::Resource.find(1)
      expect(res).to be_a Mock::Resource
    end

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).at_least_times(3)
  end

  it 'does not cache no-store resources' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      to_return(status: 200, body: {
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }.to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, no-store' # pointless, but....
      }
    )

    res = Mock::Resource.find(1)
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).once

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      with { |request| hash_excluding("If-None-Match" => "abc123") === request.headers }.
      to_return(status: 200, body: {
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }.to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, no-store'
      }
    )
    
    # no request
    Timecop.travel(10.seconds.from_now) do
      res = Mock::Resource.find(1)
      expect(res).to be_a Mock::Resource
    end

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).twice

    # do request
    Timecop.travel(60.seconds.from_now) do
      res = Mock::Resource.find(1)
      expect(res).to be_a Mock::Resource
    end

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).at_least_times(3)
  end
  
  it 'force revalidate resource' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      to_return(status: 200, body: {
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }.to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600'
      }
    )

    res = Mock::Resource.find(1)
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).once

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      with(headers: {'If-None-Match' => 'abc123'}).
      to_return(status: 304, body: "{}",
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )
    
    res = Mock::Resource.find(1, revalidate: true)
    expect(res).to be_a Mock::Resource

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).twice
  end

  it '#reload() forces revalidation of resource' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      to_return(status: 200, body: {
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }.to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600'
      }
    )

    res = Mock::Resource.find(1)
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).once

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      with(headers: {'If-None-Match' => 'abc123'}).
      to_return(status: 304, body: "{}",
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )
    
    res = res.reload
    expect(res).to be_a Mock::Resource

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).twice
  end

  it '::without_cache { }' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources/1")).
      to_return(status: 200, body: {
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }.to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600'
      }
    )

    res = Mock::Resource.without_cache do
      Mock::Resource.find(1)
    end
    expect(res).to be_a Mock::Resource

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).once

    res = Mock::Resource.find(1)
    expect(res).to be_a Mock::Resource

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources/1")).twice
  end

  it '#save() forces revalidation of resource' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources")).
      to_return(status: 200, body: [{
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }].to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600'
      }
    )

    res = Mock::Resource.first
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources")).once

    stub_request(:put, req_url(Mock::Resource.site, "/mock/resources/1")).
      to_return(status: 200, body: {
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }.to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources")).
      with(headers: {'If-None-Match' => 'abc123'}).
      to_return(status: 304, body: "{}",
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )
    
    res.save()
    res = Mock::Resource.first
    expect(res).to be_a Mock::Resource

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources")).twice
  end

  it '#destroy() forces revalidation of resource' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources")).
      to_return(status: 200, body: [{
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }].to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600'
      }
    )

    res = Mock::Resource.first
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources")).once

    stub_request(:delete, req_url(Mock::Resource.site, "/mock/resources/1")).
      to_return(status: 200, body: { }.to_json
    )

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources")).
      with(headers: {'If-None-Match' => 'abc123'}).
      to_return(status: 304, body: "{}",
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )
    
    res.destroy()
    res = Mock::Resource.first
    expect(res).to be_a Mock::Resource

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources")).twice
  end

  it '::post() forces revalidation of resource' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources")).
      to_return(status: 200, body: [{
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }].to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600'
      }
    )

    res = Mock::Resource.first
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources")).once

    stub_request(:post, req_url(Mock::Resource.site, "/mock/resources/test")).
      to_return(status: 200, body: { }.to_json
    )
    Mock::Resource.post('test', {})

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources")).
      with(headers: {'If-None-Match' => 'abc123'}).
      to_return(status: 304, body: "{}",
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )
    
    res = Mock::Resource.first
    expect(res).to be_a Mock::Resource

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources")).twice
  end

  it '::put() forces revalidation of resource' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources")).
      to_return(status: 200, body: [{
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }].to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600'
      }
    )

    res = Mock::Resource.first
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources")).once

    stub_request(:put, req_url(Mock::Resource.site, "/mock/resources/test")).
      to_return(status: 200, body: { }.to_json
    )
    Mock::Resource.put('test', {})

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources")).
      with(headers: {'If-None-Match' => 'abc123'}).
      to_return(status: 304, body: "{}",
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )
    
    res = Mock::Resource.first
    expect(res).to be_a Mock::Resource

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources")).twice
  end

  it '::patch() forces revalidation of resource' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources")).
      to_return(status: 200, body: [{
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }].to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600'
      }
    )

    res = Mock::Resource.first
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources")).once

    stub_request(:patch, req_url(Mock::Resource.site, "/mock/resources/test")).
      to_return(status: 200, body: { }.to_json
    )
    Mock::Resource.patch('test', {})

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources")).
      with(headers: {'If-None-Match' => 'abc123'}).
      to_return(status: 304, body: "{}",
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )
    
    res = Mock::Resource.first
    expect(res).to be_a Mock::Resource

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources")).twice
  end

  it '::delete() forces revalidation of resource' do
    stub_auth
    req = stub_request(:get, req_url(Mock::Resource.site, "/mock/resources")).
      to_return(status: 200, body: [{
        id: 1,
        field1: 'field-1',
        field2: 'field-2',
        field3: 'field-3'
      }].to_json,
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600'
      }
    )

    res = Mock::Resource.first
    expect(res).to be_a Mock::Resource
    remove_request_stub(req)

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources")).once

    stub_request(:delete, req_url(Mock::Resource.site, "/mock/resources/test")).
      to_return(status: 200, body: { }.to_json
    )
    Mock::Resource.delete('test', {})

    stub_request(:get, req_url(Mock::Resource.site, "/mock/resources")).
      with(headers: {'If-None-Match' => 'abc123'}).
      to_return(status: 304, body: "{}",
      headers: {
        'ETag' => 'abc123',
        'Cache-Control' => 'max-age=600, must-revalidate'
      }
    )
    
    res = Mock::Resource.first
    expect(res).to be_a Mock::Resource

    expect(WebMock).to have_requested(:get, req_url(Mock::Resource.site, "/mock/resources")).twice
  end
end

