require File.expand_path(File.dirname(__FILE__) + '/../../spec_helper')
require 'webmock/rspec'

describe "A HEL Session Resource" do
  def req_url(site, path)
    uri = URI(site.to_s)
    uri.path = path
    uri.to_s
  end

  def stub_requests
    stub_request(:post, req_url(ActiveResource::Hel::Resource.site, "/ygg/session/authenticate_by_fqda_and_password")).
      with(
        body: {fqda: ActiveResource::Hel::Resource.hel_identity, password: ActiveResource::Hel::Resource.hel_secret}
    ).to_return(
	    status: 200,
      body: {
        "id":"089af4a6-d558-40f1-86c4-c4ca3bf03815",
        "authenticated":true,
        "status":"authenticated",
        "started_at":"2019-09-21T10:15:54.000+02:00",
        "auth_credential":"Ygg::Core::Identity::Credential::ObfuscatedPassword",
        "auth_identity": ActiveResource::Hel::Resource.hel_identity,
        "auth_method":"fqda_and_password",
        "auth_confidence":"medium",
        "capabilities":["superuser","full_interface","simple_interface","api","installer"],
        "active":true,
        "msg":"Auhtentication successful!",
        "reason":"ok"
      }.to_json
    )

    stub_request(:post, req_url(ActiveResource::Hel::Resource.site, "/ygg/session/check")).
    with(
      body: {"session_id":"089af4a6-d558-40f1-86c4-c4ca3bf03815"},
      headers: {
      'Authorization'=>'Bearer 089af4a6-d558-40f1-86c4-c4ca3bf03815',
    }).
    to_return(
      status: 200, 
      body: {
        "id":"089af4a6-d558-40f1-86c4-c4ca3bf03815",
        "authenticated":true,
        "status":"authenticated",
        "started_at":"2019-09-21T10:15:54.000+02:00",
        "auth_credential":"Ygg::Core::Identity::Credential::ObfuscatedPassword",
        "auth_identity": ActiveResource::Hel::Resource.hel_identity,
        "auth_method":"fqda_and_password",
        "auth_confidence":"medium",
        "capabilities":["superuser","full_interface","simple_interface","api","installer"],
        "active":true,
        "msg":"Auhtentication successful!",
        "reason":"ok"
      }.to_json, 
      headers: {})

    stub_request(:post, req_url(ActiveResource::Hel::Resource.site, "/ygg/session/proxy_authenticate_by_fqda")).
      with(
        body: {fqda: "identity2@test.me"}
    ).to_return(
	    status: 200,
      body: {
        "id":"089af4a6-d558-40f1-86c4-c4ca3bf03815",
        "authenticated":true,
        "status":"authenticated",
        "started_at":"2019-09-21T10:15:54.000+02:00",
        "auth_credential":"Ygg::Core::Identity::Credential::ObfuscatedPassword",
        "auth_identity": "identity2@test.me",
        "auth_method":"fqda_and_password",
        "auth_confidence":"medium",
        "capabilities":["superuser","full_interface","simple_interface","api","installer"],
        "active":true,
        "msg":"Auhtentication successful!",
        "reason":"ok"
      }.to_json
    )

    stub_request(:get, req_url(ActiveResource::Hel::Resource.site, "/ygg/core/identities/identity2@test.me")).
      with(headers: {
        'Authorization'=>'Bearer 089af4a6-d558-40f1-86c4-c4ca3bf03815',
      }).
      to_return(status: 200, body: {
        qualified: "identity2@test.me"
      }.to_json, headers: {})
  end

  describe "regarding proxied sessions" do
    it 'should check the current session' do
      stub_requests
      Ygg::Session.cached = false
      Ygg::Session.connection.authenticate!

      s = Ygg::Session.check_validity(Ygg::Session.connection.session)
      expect(s).to be_truthy
      expect(s['active']).to be_truthy

    end

    it 'should be able to get a session for another identity' do
      stub_requests
      identity = "identity2@test.me"

      s = Ygg::Session.new_for_identity(identity)
      expect(s).to be_authenticated
      expect(s).to be_a_kind_of Ygg::Session
      expect(s.identity.qualified).to eq identity
    end

#    non funziona e non capisco
#    it 'should be able to act as another identity' do
#      identity = $INTEGRATION_CONFIG['normal']['identity']
#
#      s = Ygg::Session.new_for_identity(identity)
#
#      ActiveResource::Hel::Resource.with_session(s) do
#        Ygg::Core::Identity.find_by_qualified(identity)
#        connection =  Ygg::Core::Identity.connection
#        connection.session.should be_authenticated
#        connection.session.identity.should == identity
#      end
#    end

  end
end
